using HiTeachCC.Model.Core.Dtos;
using HiTeachCC.Model.Core.Models;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using TEAMModelOS.SDK.Extension.JwtAuth.Models;

namespace HiTeachCC.Service.Core.Interface
{
    public interface ILoginInfoService : IBusinessService, IBaseService
    {
        Task<object> VerifyWeChat(Dictionary<string, object> @params);
        Dictionary<string, object> GetWechatConfig(string Time);
        Task<MiniAPPLoginInfo> VerifyMiniAPP(Dictionary<string, object> dict);
        Task<MiniAPPToken> GetMiniAPPAccessToken();
        Task<JwtResponse> MiniAPPLogin(Dictionary<string, object> @params);
        Task<JwtResponse> TeamModelLogin(TicketInfo ticket);
    }
}
